global function FSU_init

global function FSU_RegisterCommand

global function FSU_CanCreatePoll
global function FSU_CreatePoll
global function FSU_GetPollResultIndex


struct commandStruct
{
  string usage
  void functionref( entity, array < string > ) func
}

// List of registered commands
table < string, commandStruct > commands

// Poll stuff
// Used to check if a poll is on
array < string > poll_options
// other poll stuff
table < entity, int > votes
float poll_time
float poll_start
string poll_before
int poll_result


// init
void function FSU_init ()
{
  AddCallback_OnClientConnected ( OnClientConnected )
  
  
  // Register commands

  FSU_RegisterCommand( "!help", "\x1b[113m!help <page>\x1b[0m Lists registered commands", FSU_C_Help )
  FSU_RegisterCommand( "!name", "\x1b[113m!name\x1b[0m Returns the name of the current server", FSU_C_Name )
  FSU_RegisterCommand( "!owner", "\x1b[113m!owner\x1b[0m Returns the name of the owner if provided", FSU_C_Owner )
  FSU_RegisterCommand( "!mods", "\x1b[113m!mods\x1b[0m Lists installed mods on server. This list has to be manually updated and may not be uptodate!", FSU_C_Mods )
    
  FSU_RegisterCommand( "!vote", "\x1b[113m!vote\x1b[0m <number> Allows you to vote on polls", FSU_C_Vote )
  FSU_RegisterCommand( "!usage", "\x1b[113m!usage\x1b[0m <command> Prints usage of provided command", FSU_C_Usage )
  
  AddCallback_OnReceivedSayTextMessage( CheckForCommand )
}

// callbacks
void function OnClientConnected ( entity player )
{
  string finalMessage = fsu_welcome_message
  
  if( fsu_w_enable_owner )
    finalMessage += "\nHosted by: \x1b[113m\"" + fsu_owner + "\"\x1b[0m"
  
  if( fsu_w_enable_commands )
  {
    finalMessage += "\nCommands:"
    foreach ( cmd, func in commands )
      finalMessage += "\n    \x1b[113m-\x1b[0m " + cmd
  }
  
  Chat_ServerPrivateMessage( player, finalMessage, false )
  
  // If a poll is active try to show it to late joiners
  // This means we need to calc new length
  if ( FSU_CanCreatePoll() )
    return
  
  string poll_text
  foreach ( _index, line in poll_options )
    poll_text += _index + 1 + ". " + line + "\n"
  
  SendHudMessage( player, poll_before + "\n" + poll_text , 0.005, 0.3, 240, 180, 40, 230, 0.2, poll_time, 0.2)
}

ClServer_MessageStruct function CheckForCommand( ClServer_MessageStruct message )
{
  // Unsure if needed, better to check before we do anything funny
  if ( message.message.len() == 0)
    return message
  
  // Check if message starts wit prefix
  if ( message.message.find( "!" ) != 0 )
    return message
  
  // We now confirmed the user is trying to run a command
  // Dont show their message to anyone
  message.shouldBlock = true
  
  
  array < string > splitMsg = split( message.message, " " )
  
  string command = splitMsg[0].tolower()
  array < string > args
  foreach ( _index, string arg in splitMsg )
    if ( _index != 0 )
      args.append( arg )
  
  // Try to execute cammand
  if( command in commands )
  {
    if( commands[ command ].func != null )
      commands[ command ].func( message.player, args )
  }
  else
    Chat_ServerPrivateMessage( message.player, "Unknown command: \x1b[113m\"" + message.message + "\"\x1b[0m", false )
  
  
  printt("[FSU]", message.player.GetPlayerName(), message.player.GetUID(), "ran command \"" + message.message +"\"")
  return message
}

// command register
void function FSU_RegisterCommand ( string command, string usage, void functionref( entity, array < string > ) callbackFunc )
{
  if ( command != command.tolower() )
    throw "[FSU] Registered commands have to be lowercase!"
  
  commandStruct _command
  _command.usage = usage
  _command.func = callbackFunc
  commands[ command ] <- _command
  printt( "[FSU] Registered command:", command )
}

// poll stuff
//public
bool function FSU_CanCreatePoll ()
{
  return poll_options.len() == 0 ? true : false
}

void function FSU_CreatePoll ( array < string > options, string before, float duration )
{
  if ( options.len() > 7 )
    printt("[FSU] Polls larger than 7 may interfere with chat box!")
  
  poll_start = Time()
  poll_time = duration
  poll_before = before
  
  foreach( option in options )
    poll_options.append( option )
  
  string poll_text
  foreach ( _index, line in poll_options )
    poll_text += _index + 1 + ". " + line + "\n"
  
  
  foreach( player in GetPlayerArray() )
    SendHudMessage( player, poll_before + "\n" + poll_text , 0.005, 0.3, 240, 180, 40, 230, 0.2, poll_time, 0.2)
  
  thread FSU_PollEndTimeWatcher_Threaded ()
}

int function FSU_GetPollResultIndex ()
{
  return poll_result
}
// private

void function FSU_PollEndTimeWatcher_Threaded ()
{
  wait poll_time
  
  // Count votes
  if ( votes.len() == 0 )
  {
    poll_result = -1
    return
  }
  
  array < int > votes_counted
  foreach ( option in poll_options )
    votes_counted.append(0)
  
  
  foreach ( player, index in votes )
    votes_counted[ index - 1 ]++
  
  int poll_result_votes
  // Could sort but this is prob shorter
  foreach ( _index, count in votes_counted )
  {
    if ( poll_result_votes < count )
    {
      poll_result_votes = count
      poll_result = _index
    }
  }
  
  Chat_ServerBroadcast( "Poll ended! \x1b[113m" + poll_options[ FSU_GetPollResultIndex() ] + "\x1b[0m won!"  )
  
  poll_options.clear()
  votes.clear()
}

// !help
void function FSU_C_Help ( entity player, array < string > args )
{
  string returnMessage
  foreach ( cmd, func in commands )
    returnMessage += "\n    \x1b[113m-\x1b[0m " + cmd
  
  Chat_ServerPrivateMessage( player, "Commands: " + returnMessage, false )
  
  return
}

// !owner
void function FSU_C_Owner ( entity player, array < string > args )
{
  Chat_ServerPrivateMessage( player, "Owner: \x1b[113m\"" + fsu_owner + "\"\x1b[0m", false )
  
  return 
}

// !name
void function FSU_C_Name ( entity player, array < string > args )
{
  Chat_ServerPrivateMessage( player, "Server name: \x1b[113m\"" + GetConVarString("ns_server_name") + "\"\x1b[0m", false )
  
  return
}

// !mods
void function FSU_C_Mods ( entity player, array < string > args )
{
  string returnMessage
  foreach ( mod in fsu_mods )
    returnMessage += "\n    \x1b[113m-\x1b[0m " + mod
  
  Chat_ServerPrivateMessage( player, "Mods:" + returnMessage, false )
  
  return
}

// !vote
void function FSU_C_Vote ( entity player, array < string > args )
{
  if ( FSU_CanCreatePoll() )
  {
    Chat_ServerPrivateMessage( player, "No vote currently open!", false )
    return
  }
  
  if ( player in votes )
  {
    Chat_ServerPrivateMessage( player, "You have already voted!", false )
    return
  }
  
  if ( args.len() == 0 )
  {
    Chat_ServerPrivateMessage( player, "Missing argument!", false )
    return
  }
  
  if ( !IsValidVoteInt( args[0], poll_options.len() ) )
  {
    Chat_ServerPrivateMessage( player, "Invalid argument!", false )
    return
  }
  
  int index = args[0].tointeger()
  
  votes[ player ] <- index
  
  
  Chat_ServerPrivateMessage( player, "You have voted for \x1b[113m" + poll_options[ index - 1 ] + "\x1b[0m", false )
}

void function FSU_C_Usage ( entity player, array < string > args )
{
  if ( args.len() == 0 )
  {
    Chat_ServerPrivateMessage( player, "Usage: !usage <command>", false )
    return
  }
  
  string cmd = "!" + args[0].tolower()
  
  if ( cmd in commands )
    Chat_ServerPrivateMessage( player, "Usage: " + commands[ cmd ].usage, false )
  else
    Chat_ServerPrivateMessage( player, "Unknown command passed in!", false )
}


bool function IsValidVoteInt( string arg, int max )
{
  int _index = arg.tointeger()
  
  if ( _index < 1 )
    return false
  
  if ( _index > max )
    return false
  
  return true
}