global function FSU_Map_init

array < entity > playersWantingToChangeMap
array < string > map_list
string next_map



void function FSU_Map_init ()
{
  if ( fsu_enable_rtv )
    FSU_RegisterCommand( "!rtv","\x1b[113m!rtv\x1b[0m If enough players vote a poll to change map will start", FSU_C_Rtv )
  
  
  if ( GetMapName() != "mp_lobby" )
  {
    thread RockTheVoteMapChange_Threaded ()
    if ( fsu_enable_map_vote_before_match_end )
    {
      AddCallback_GameStateEnter( eGameState.Postmatch, EndOfMatchMapChange_Threaded )
      AddCallback_GameStateEnter( eGameState.Playing, EndOfMatchMapVote_Threaded )
    }
  }
}


void function EndOfMatchMapVote_Threaded ()
{
  float end_time = expect float ( GetServerVar("gameEndTime") )
  end_time -= fsu_vote_time_before_postmatch
  
  
  while ( Time() < end_time ) 
    WaitFrame()
  
  if ( !FSU_CanCreatePoll() )
    return
  
  Chat_ServerBroadcast( "Map voting has began! Use \x1b[113m\"!vote [number]\"\x1b[0m to vote" ) 
  
  CreateMapsSelection()
  FSU_CreatePoll( GetFormattedMapsArray(), "Next map vote", 20 )
  
  wait 20
  
  
  if ( FSU_GetPollResultIndex() == -1 )
  {
    Chat_ServerBroadcast( "No votes cast! Map will change based on playlist!" )
    return
  }
  
  next_map = map_list[ FSU_GetPollResultIndex() ]
}

void function EndOfMatchMapChange_Threaded ()
{
  wait GAME_POSTMATCH_LENGTH - 1
  
  
  if ( next_map != "")
    GameRules_ChangeMap( next_map, GameRules_GetGameMode() )
}

void function RockTheVoteMapChange_Threaded ()
{
  while ( playersWantingToChangeMap.len() < GetPlayerArray().len() * fsu_mapchange_fraction || GetGameState() != eGameState.Playing )
    WaitFrame()
  
  
  Chat_ServerBroadcast( "Map voting has began! Use \x1b[113m\"!vote [number]\"\x1b[0m to vote" ) 
  
  CreateMapsSelection()
  FSU_CreatePoll( GetFormattedMapsArray(), "Next map vote", 20 )
  
  wait 20
  
  
  if ( FSU_GetPollResultIndex() == -1 )
  {
    Chat_ServerBroadcast( "No votes cast! You can call \x1b[113m!rtv\x1b[0m again to vote!" ) 
    
    playersWantingToChangeMap.clear()
    map_list.clear()
    
    thread RockTheVoteMapChange_Threaded()
    return
  }
  
  next_map = map_list[ FSU_GetPollResultIndex() ]
  
  Chat_ServerBroadcast( "\x1b[113m" + fsu_l_maps[ next_map ] + "\x1b[0m won! Changing map in 5s" ) 
  
  wait 5
  
  GameRules_ChangeMap( next_map, GameRules_GetGameMode() )
}

// !rtv
void function FSU_C_Rtv ( entity player, array < string > args )
{
  if ( GetMapName() == "mp_lobby" )
  {
    Chat_ServerPrivateMessage( player, "Can't rtv in lobby", false )
    return
  }
  
  if ( GetGameState() != eGameState.Playing )
  {
    Chat_ServerPrivateMessage( player, "Can't rtv in this game state", false )
    return
  }
  
  
  if ( playersWantingToChangeMap.find( player ) != -1 )
  {
    Chat_ServerPrivateMessage( player, "You have already voted!", false )
    return
  }
  
  playersWantingToChangeMap.append( player )
  
  Chat_ServerBroadcast( "\x1b[113m[" + playersWantingToChangeMap.len() + "/" + int ( GetPlayerArray().len() * fsu_mapchange_fraction ) + "]\x1b[0m players want to change map (!rtv)" ) 
  
  
  return
}


void function CreateMapsSelection()
{
  array < string > maps = clone fsu_mapvote_array
  
  // randomise the array
  for ( int i = 0; i < 7; i++ )
  {
    string map = maps[ RandomInt ( maps.len() ) ]
    if ( map_list.find( map ) != -1 )
    {
      i--
      continue
    }
    
    map_list.append( map )
  }
}

array < string > function GetFormattedMapsArray()
{
  array < string > formatted_array
  foreach ( map in map_list )
    formatted_array.append( fsu_l_maps[ map ] )
  
  return formatted_array
}